package fr.sanchezm.attestationsCovid19.ui.add

import android.annotation.SuppressLint
import android.os.Build
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import fr.sanchezm.attestationsCovid19.R
import fr.sanchezm.attestationsCovid19.data.db.entity.Attestation
import fr.sanchezm.attestationsCovid19.data.db.entity.Profile
import fr.sanchezm.attestationsCovid19.data.repository.AttestationRepository
import fr.sanchezm.attestationsCovid19.data.repository.ConfigRepository
import fr.sanchezm.attestationsCovid19.data.repository.ProfileRepository
import fr.sanchezm.attestationsCovid19.utilities.Event
import java.text.SimpleDateFormat
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*

class AddViewModel(
    private val configRepository: ConfigRepository,
    private val profileRepository: ProfileRepository,
    private val attestationRepository: AttestationRepository
) : ViewModel() {

    private val _errorMessage = MutableLiveData<Event<Int>>()
    private val _dataMessage = MutableLiveData<Event<Int>>()
    private val _startActivity = MutableLiveData<Event<Long>>()

    val errorMessage: LiveData<Event<Int>>
        get() = _errorMessage

    val dataMessage: LiveData<Event<Int>>
        get() = _dataMessage

    val startActivity: LiveData<Event<Long>>
        get() = _startActivity

    // region ViewField
    val firstName = MutableLiveData<String>()
    val lastName = MutableLiveData<String>()
    val birthday = MutableLiveData<String>()
    val birthPlace = MutableLiveData<String>()
    val address = MutableLiveData<String>()
    val city = MutableLiveData<String>()
    val postalCode = MutableLiveData<String>()
    val exitDate = MutableLiveData<String>()
    val exitHour = MutableLiveData<String>()

    val reason1 = MutableLiveData(false)
    val reason2 = MutableLiveData(false)
    val reason3 = MutableLiveData(false)
    val reason4 = MutableLiveData(false)
    val reason5 = MutableLiveData(false)
    val reason6 = MutableLiveData(false)
    val reason7 = MutableLiveData(false)
    // endregion

    private val datePattern = "dd/MM/yyyy"
    private val timePattern = "HH'h'mm"

    @SuppressLint("LongLogTag")
    fun onGenerateAttestationClick() {
        if (checkAllValue()) {
            val attestation = getAttestation()

            profileRepository.insertProfile(getProfileFromView())
            attestationRepository.addAttestation(attestation)

            _startActivity.value = Event(attestation.createAt)
        } else {
            _errorMessage.value = Event(R.string.error_cannot_create_attestation)
            Log.e("onGenerateAttestationClick", "Cannot generate Attestation")
        }
    }

    init {
        setProfileValue()
        setDateHourToday()
    }

    private fun setProfileValue() {
        val profile = profileRepository.getProfile()

        firstName.value = profile.value?.firstName
        lastName.value = profile.value?.lastName
        birthday.value = profile.value?.birthday
        birthPlace.value = profile.value?.birthPlace
        address.value = profile.value?.address
        city.value = profile.value?.city
        postalCode.value = profile.value?.postalCode
    }

    private fun setDateHourToday() {
        exitDate.value = getDateOrTimeFromPattern(datePattern)
        exitHour.value = getDateOrTimeFromPattern(timePattern)
    }

    private fun getDateOrTimeFromPattern(pattern: String): String {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            DateTimeFormatter.ofPattern(pattern).format(LocalDateTime.now())
        } else {
            SimpleDateFormat(pattern, Locale.FRANCE).format(Date())
        }
    }

    private fun getProfileFromView(): Profile {
        return Profile(
            firstName.value.toString(),
            lastName.value.toString(),
            birthday.value.toString(),
            birthPlace.value.toString(),
            address.value.toString(),
            city.value.toString(),
            postalCode.value.toString()
        )
    }

    private fun getAttestation(): Attestation {
        return Attestation(
            getProfileFromView(),
            exitDate.value.toString(),
            exitHour.value.toString(),
            Date().time,
            getReasons()
        )
    }

    private fun getReasons(): List<Int> {
        val reasons = ArrayList<Int>()

        if (reason1.value!!) reasons.add(1)
        if (reason2.value!!) reasons.add(2)
        if (reason3.value!!) reasons.add(3)
        if (reason4.value!!) reasons.add(4)
        if (reason5.value!!) reasons.add(5)
        if (reason6.value!!) reasons.add(6)
        if (reason7.value!!) reasons.add(7)

        return reasons
    }

    private fun checkAllValue(): Boolean {
        return !firstName.value.isNullOrEmpty()
                && !lastName.value.isNullOrEmpty()
                && !birthday.value.isNullOrEmpty()
                && !birthPlace.value.isNullOrEmpty()
                && !address.value.isNullOrEmpty()
                && !city.value.isNullOrEmpty()
                && !postalCode.value.isNullOrEmpty()
                && !exitDate.value.isNullOrEmpty()
                && !exitHour.value.isNullOrEmpty()
                && checkAtLeastOneReasonSelected()
    }

    private fun checkAtLeastOneReasonSelected(): Boolean {
        return reason1.value!!
                || reason2.value!!
                || reason3.value!!
                || reason4.value!!
                || reason5.value!!
                || reason6.value!!
                || reason7.value!!
    }

}